@@ -335,8 +335,6 @@ impl FlatFiles {
         path_separator: String,
         schema_titles: String,
     ) -> Result<Self> {
-        smartstring::validate();
-
         let output_path = PathBuf::from(output_dir);
         if output_path.is_dir() {
             if force {
@@ -479,8 +477,15 @@ impl FlatFiles {
                         .iter()
                         .map(|val| (val.as_str().unwrap().to_string())) //value known as str
                         .collect();
-                    let new_value = json!(keys.join(","));
-                    to_insert.push((key.clone(), new_value))
+                    let mut csv_bytes = vec![];
+                    {
+                        let mut csv_writer = csv::Writer::from_writer(&mut csv_bytes);
+                        csv_writer.write_record(keys).unwrap(); // safe as just writing to vec
+                    }
+                    let values = String::from_utf8_lossy(&csv_bytes);
+                    let trimmed = values.trim();
+                    let trimmed_value = json!(trimmed);
+                    to_insert.push((key.clone(), trimmed_value))
                 } else if obj_count == arr_length {
                     to_delete.push(key.clone());
                     let mut removed_array = value.take(); // obj.remove(&key).unwrap(); //key known
@@ -1014,25 +1019,45 @@ impl FlatFiles {
                 continue;
             }
             let table_order = metadata.order.clone();
+            let mut foreign_keys = vec![];
 
             for order in table_order {
                 if metadata.ignore_fields[order] {
                     continue;
                 }
+                let data_type = match metadata.field_type[order].as_str() {
+                    "text" => "string",
+                    "date" => "datetime",
+                    rest => &rest
+                };
+
+                let field_name = &metadata.fields[order];
+                let field_title = &metadata.fields[order];
+
                 let field = json!({
-                    "name": metadata.field_titles[order],
-                    "type": metadata.field_type[order],
+                    "name": field_name,
+                    "type": data_type,
                     "count": metadata.field_counts[order],
                 });
                 fields.push(field);
+
+                if field_name.starts_with("_link") && field_name != "_link" {
+                    let foreign_table = self.table_order.get(&field_name[6..]).unwrap();
+                    foreign_keys.push(
+                        json!(
+                        {"fields":field_title, "reference": {"resource": foreign_table, "fields": "_link"}}
+                    ))
+                };
             }
 
+
             let mut resource = json!({
                 "profile": "tabular-data-resource",
-                "name": table_title,
+                "name": table_title.to_lowercase(),
                 "schema": {
                     "fields": fields,
-                    "primaryKey": "_link"
+                    "primaryKey": "_link",
+                    "foreignKeys": foreign_keys
                 }
             });
             if self.csv {
@@ -1500,13 +1525,23 @@ impl FlatFiles {
         conn.execute_batch(
             "PRAGMA journal_mode = OFF;
              PRAGMA synchronous = 0;
-             PRAGMA cache_size = 1000000;
              PRAGMA locking_mode = EXCLUSIVE;
              PRAGMA temp_store = MEMORY;",
         )
         .context(RusqliteSnafu {})?;
 
-        for (table_name, table_title) in self.table_order.iter() {
+
+        let mut new_table_order = self.table_order.keys()
+            .cloned()
+            .filter(|table| table != &self.main_table_name)
+            .collect_vec();
+
+        new_table_order.sort_by(|a, b| {a.len().cmp(&b.len())});
+
+        new_table_order.insert(0,self.main_table_name.clone());
+
+        for table_name in new_table_order.iter() {
+            let table_title = self.table_order.get(table_name).unwrap();
             let mut output_row = Vec::new();
             let metadata = self.table_metadata.get(table_name).unwrap();
             if metadata.rows == 0 || metadata.ignore {
@@ -1520,22 +1555,42 @@ impl FlatFiles {
             let table_order = metadata.order.clone();
             let mut create_table_sql = String::new();
             create_table_sql.push_str(&format!(
-                "CREATE TABLE \"{ }\"(",
+                "CREATE TABLE \"{ }\"(\n",
                 table_title.to_lowercase()
             ));
 
             let mut fields = Vec::new();
+            let mut indexes = Vec::new();
+            let mut foreign_keys = Vec::new();
+
             for order in table_order {
                 if metadata.ignore_fields[order] {
                     continue;
                 }
+                let primary_key = if metadata.fields[order] == "_link" {" PRIMARY KEY"} else {""};
+
                 fields.push(format!(
-                    "    \"{}\" {}",
+                    "    \"{}\" {}{}",
                     metadata.field_titles_lc[order],
-                    postgresql::to_postgresql_type(&metadata.field_type[order])
+                    postgresql::to_postgresql_type(&metadata.field_type[order]),
+                    primary_key
                 ));
+                let field_name =  &metadata.fields[order];
+                let field_title =  &metadata.field_titles_lc[order];
+
+                if field_name.starts_with("_link") && field_name != "_link" {
+                    indexes.push(format!("CREATE INDEX idx_{table_title}_{field_title} on {table_title}({field_title});"));
+                    let foreign_table = self.table_order.get(&field_name[6..]).unwrap();
+                    foreign_keys.push(format!("FOREIGN KEY ({field_title}) REFERENCES {foreign_table}(_link)"));
+                }
             }
-            create_table_sql.push_str(&format!("{});\n\n", fields.join(",\n")));
+
+            let comma = if foreign_keys.is_empty() {""} else {",\n"};
+
+            create_table_sql.push_str(&format!("{}\n{}{});\n", fields.join(",\n"), comma, foreign_keys.join(",\n")));
+
+            create_table_sql.push_str(&format!("{}\n\n", indexes.join("\n")));
+
             log::debug!("create table sql: {create_table_sql}");
             conn.execute(&create_table_sql, [])
                 .context(RusqliteSnafu {})?;
@@ -1605,7 +1660,6 @@ fn value_convert(
     num: usize,
     date_re: &Regex,
 ) -> String {
-    //let value_type = output_fields.get("type");
     let value_type = &field_type[num];
 
     match value {
